package ipm.ipmworks.service;

import com.mongodb.BasicDBObject;
import com.mongodb.client.MongoCursor;
import com.mongodb.client.MongoIterable;
import com.mongodb.client.gridfs.GridFSFindIterable;
import com.mongodb.client.gridfs.model.GridFSFile;
import org.bson.types.ObjectId;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.core.io.InputStreamResource;
import org.springframework.data.mongodb.core.query.Criteria;
import org.springframework.data.mongodb.core.query.Query;
import org.springframework.data.mongodb.gridfs.GridFsResource;
import org.springframework.data.mongodb.gridfs.GridFsTemplate;
import org.springframework.stereotype.Service;
import org.springframework.web.multipart.MultipartFile;

import java.io.IOException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

@Service
public class FileService {

    @Autowired
    GridFsTemplate gridFsTemplate;

    public GridFsResource getFileById(String id) throws IOException {
        GridFSFile gridFSFile = gridFsTemplate.findOne(new Query(Criteria.where("_id").is(id)));
        if (gridFSFile != null) {
            return gridFsTemplate.getResource(gridFSFile);
        }else{
            return null;
        }
    }

    public GridFsResource getResourceImage(String idResource,String type) {
        Query query = new Query();
        query.addCriteria(Criteria.where("metadata.idResource").is(idResource).and("metadata.type").is(type));
        GridFSFile gridFSFile = gridFsTemplate.findOne(query);
        if (gridFSFile == null) return null;
        return gridFsTemplate.getResource(gridFSFile);
    }

    public List<Map> getFileDescription(String type){
        List fileDescList = new ArrayList();
        Query query = new Query();
        query.addCriteria(Criteria.where("metadata.type").is(type));
        GridFSFindIterable gridFSFileList = gridFsTemplate.find(query);
        MongoCursor<GridFSFile> iterator = gridFSFileList.iterator();
        while (iterator.hasNext()){
            GridFSFile gfs = iterator.next();
            String description = (String)gfs.getMetadata().get("description");
            String fileIdentifier = gfs.getObjectId().toHexString();
            Map m = new HashMap();
            m.put("description",description);
            m.put("fileIdentifier",fileIdentifier);
            fileDescList.add(m);
        }
        return fileDescList;
    }

    public boolean uploadFile(MultipartFile file,String description,String type) throws IOException {
        if (file != null) {
            BasicDBObject metaDataResource = new BasicDBObject();
            metaDataResource.put("type", type);
            metaDataResource.put("description", description);
            gridFsTemplate.store(file.getInputStream(), file.getOriginalFilename(), file.getContentType(), metaDataResource);
            return true;
        } else {
            return false;
        }
    }

    public boolean deleteFile(String id){
        gridFsTemplate.delete(new Query(Criteria.where("_id").is(id)));
        return true;
    }
    public boolean deleteFile(String id, String fileName){
        gridFsTemplate.delete(new Query(Criteria.where("f").is(id)));
        return true;
    }
}
